@@ -340,6 +340,7 @@ gnome_keyring_application_ref_new_from_pid (pid_t pid)
 	app_ref = g_new0 (GnomeKeyringApplicationRef, 1);
 
 #if defined(__linux__) || defined(__FreeBSD__)
+	g_assert (pid > 0);
 	{
 		char *buffer;
 		int len;
@@ -441,13 +442,24 @@ request_allowed_for_app (GnomeKeyringAccessRequest *request,
 		}
 		/* TODO: verify app ACL vs keyring?? */
 		return TRUE;
+
 	} else if (request->item != NULL) {
 		if (request->item->locked) {
 			return FALSE;
 		}
+
+		/* Is it only basic info read access (no secret)? */
+		if(request->request_type == GNOME_KEYRING_ACCESS_REQUEST_ITEM && 
+		   request->access_type == GNOME_KEYRING_ACCESS_READ) {	
+
+			/* If item is not marked as an application secret then automatic access */
+			if ((request->item->type & GNOME_KEYRING_ITEM_APPLICATION_SECRET) == 0)
+				return TRUE;
+		}
+			
+		/* Otherwise full access to secret, or modify access we check the calling application */
 		for (l = request->item->acl; l != NULL; l = l->next) {
 			ac = l->data;
-			
 			if (app_ref_match (app_ref, ac->application) &&
 			    (ac->types_allowed & request->access_type) == request->access_type) {
 				return TRUE;
@@ -538,12 +550,23 @@ match_attributes (GnomeKeyringItem *item,
 }
 
 static GnomeKeyringAccessRequest *
-access_request_from_item (GnomeKeyringItem *item,
-			  GnomeKeyringAccessType access_type)
+access_request_from_item (GnomeKeyringItem *item)
 {
 	GnomeKeyringAccessRequest *access_request;
 	access_request = g_new0 (GnomeKeyringAccessRequest, 1);
 	access_request->request_type = GNOME_KEYRING_ACCESS_REQUEST_ITEM;
+	access_request->access_type = GNOME_KEYRING_ACCESS_READ; /* Always only read access */
+	access_request->item = item;
+	return access_request;
+}
+
+static GnomeKeyringAccessRequest *
+access_request_from_item_with_secret (GnomeKeyringItem *item,
+			  	      GnomeKeyringAccessType access_type)
+{
+	GnomeKeyringAccessRequest *access_request;
+	access_request = g_new0 (GnomeKeyringAccessRequest, 1);
+	access_request->request_type = GNOME_KEYRING_ACCESS_REQUEST_ITEM_SECRET;
 	access_request->access_type = access_type;
 	access_request->item = item;
 	return access_request;
@@ -1256,8 +1279,8 @@ op_list_items_execute (GString *packet,
 	char *keyring_name;
 	GnomeKeyringOpCode opcode;
 	GnomeKeyringItem *item;
-	GList *l;
-	GnomeKeyringAccessRequest *req;
+	GList *l, *items;
+	GnomeKeyringAccessRequest *req, *list_req;
 	
 	if (!gnome_keyring_proto_decode_op_string (packet,
 						   &opcode,
@@ -1284,11 +1307,32 @@ op_list_items_execute (GString *packet,
 			gnome_keyring_proto_add_uint32 (result, 0);
 		} else {
 			gnome_keyring_proto_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
-			gnome_keyring_proto_add_uint32 (result, g_list_length (keyring->items));
+
+			/* 
+			 * Request access to these items. This cannot be done earlier in 
+			 * op_list_items_collect because the keyring may have been locked.
+			 */
+			list_req = access_request_from_item (NULL);
+			
+			items = NULL;
 			for (l = keyring->items; l != NULL; l = l->next) {
+				list_req->item = l->data;
+				list_req->granted = FALSE;
+				if (request_allowed_for_app (list_req, app_ref, NULL))
+					items = g_list_prepend (items, list_req->item);
+			}
+			items = g_list_reverse (items);
+
+			gnome_keyring_access_request_free (list_req);
+
+			/* Send the results */			
+			gnome_keyring_proto_add_uint32 (result, g_list_length (items));
+			for (l = items; l != NULL; l = l->next) {
 				item = l->data;
 				gnome_keyring_proto_add_uint32 (result, item->id);
 			}
+
+			g_list_free (items);
 		}
 	}
 	
@@ -1340,11 +1384,11 @@ op_create_item_collect (GString *packet,
 
 		for (ilist = keyring->items; ilist != NULL; ilist = ilist->next) {
 			item = ilist->data;
-			if (item->type == type &&
+			if ((item->type & GNOME_KEYRING_ITEM_TYPE_MASK) == (type & GNOME_KEYRING_ITEM_TYPE_MASK) &&
 			    match_attributes (item, keyring->locked ? hashed : attributes, TRUE)) {
 				found_existing = TRUE;
 				access_request =
-					access_request_from_item (item, GNOME_KEYRING_ACCESS_WRITE);
+					access_request_from_item_with_secret (item, GNOME_KEYRING_ACCESS_WRITE);
 				access_requests = g_list_prepend (access_requests,
 								  access_request);
 				break;
@@ -1419,7 +1463,7 @@ op_create_item_execute (GString *packet,
 		goto bail;
 	}
 
-	if (type >= GNOME_KEYRING_ITEM_LAST_TYPE ||
+	if ((type & GNOME_KEYRING_ITEM_TYPE_MASK) >= GNOME_KEYRING_ITEM_LAST_TYPE ||
 	    display_name == NULL ||
 	    secret == NULL) {
 		res = GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
@@ -1444,6 +1488,9 @@ op_create_item_execute (GString *packet,
 		goto bail;
 	}
 
+	/* Copy in item type flags */
+	item->type |= (type & ~GNOME_KEYRING_ITEM_TYPE_MASK);
+
 	g_free (item->display_name);
 	item->display_name = g_strdup (display_name);
 	g_free (item->secret);
@@ -1500,7 +1547,7 @@ op_delete_item_collect (GString *packet,
 			item = find_item (keyring, item_id);
 			if (item != NULL) {
 				access_request =
-					access_request_from_item (item,
+					access_request_from_item_with_secret (item,
 								  GNOME_KEYRING_ACCESS_REMOVE);
 				access_requests = g_list_prepend (access_requests,
 								  access_request);
@@ -1575,22 +1622,18 @@ op_delete_item_execute (GString *packet,
 
 
 static gboolean
-op_get_item_info_or_attributes_collect (GString *packet,
-					GList **access_requests_out)
+op_get_item_info_collect (GString *packet, GList **access_requests_out)
 {
 	char *keyring_name;
 	GnomeKeyring *keyring;
 	GnomeKeyringItem *item;
 	GnomeKeyringOpCode opcode;
-	guint32 item_id;
+	guint32 item_id, flags;
 	GnomeKeyringAccessRequest *access_request;
 	GList *access_requests;
 
-	
-	if (!gnome_keyring_proto_decode_op_string_int (packet,
-						       &opcode,
-						       &keyring_name,
-						       &item_id)) {
+	if (!gnome_keyring_proto_decode_get_item_info (packet, &opcode, &keyring_name, 
+						       &item_id, &flags)) {
 		return FALSE;
 	}
 
@@ -1600,9 +1643,13 @@ op_get_item_info_or_attributes_collect (GString *packet,
 		if (keyring != NULL) {
 			item = find_item (keyring, item_id);
 			if (item != NULL) {
-				access_request =
-					access_request_from_item (item,
-								  GNOME_KEYRING_ACCESS_READ);
+				/* Request access based on what parts were desired */
+				if ((flags & GNOME_KEYRING_ITEM_INFO_SECRET) == GNOME_KEYRING_ITEM_INFO_SECRET) {
+					access_request = access_request_from_item_with_secret (item,
+									  GNOME_KEYRING_ACCESS_READ);
+				} else {
+					access_request = access_request_from_item (item);
+				}
 				access_requests = g_list_prepend (access_requests,
 								  access_request);
 			}
@@ -1622,17 +1669,15 @@ op_get_item_info_execute (GString *packet,
 			  GnomeKeyringApplicationRef *app_ref,
 			  GList *access_requests)
 {
-	char *keyring_name;
+	char *keyring_name, *secret;
 	GnomeKeyring *keyring;
 	GnomeKeyringItem *item;
 	GnomeKeyringOpCode opcode;
-	guint32 item_id;
+	guint32 item_id, flags;
 	GnomeKeyringAccessRequest *access_request;
 	
-	if (!gnome_keyring_proto_decode_op_string_int (packet,
-						       &opcode,
-						       &keyring_name,
-						       &item_id)) {
+	if (!gnome_keyring_proto_decode_get_item_info (packet, &opcode, &keyring_name,
+						       &item_id, &flags)) {
 		return FALSE;
 	}
 
@@ -1667,9 +1712,19 @@ op_get_item_info_execute (GString *packet,
 	if (!gnome_keyring_proto_add_utf8_string (result, item->display_name)) {
 		return FALSE;
 	}
-	if (!gnome_keyring_proto_add_utf8_string (result, item->secret)) {
+
+	/* Only return the secret if it was requested */
+	secret = NULL;
+	if ((flags & GNOME_KEYRING_ITEM_INFO_SECRET) == GNOME_KEYRING_ITEM_INFO_SECRET) {
+		g_assert(access_request->request_type == GNOME_KEYRING_ACCESS_REQUEST_ITEM_SECRET);
+		secret = item->secret;
+	}
+
+	/* Always put the secret string or NULL in the results for compatibility */
+	if (!gnome_keyring_proto_add_utf8_string (result, secret)) {
 		return FALSE;
 	}
+
 	gnome_keyring_proto_add_time (result, keyring->mtime);
 	gnome_keyring_proto_add_time (result, keyring->ctime);
 	
@@ -1679,6 +1734,46 @@ out:
 	return TRUE;
 }
 
+static gboolean
+op_get_item_acl_or_attributes_collect (GString *packet,
+					GList **access_requests_out)
+{
+	char *keyring_name;
+	GnomeKeyring *keyring;
+	GnomeKeyringItem *item;
+	GnomeKeyringOpCode opcode;
+	guint32 item_id;
+	GnomeKeyringAccessRequest *access_request;
+	GList *access_requests;
+
+	
+	if (!gnome_keyring_proto_decode_op_string_int (packet,
+						       &opcode,
+						       &keyring_name,
+						       &item_id)) {
+		return FALSE;
+	}
+
+	access_requests = NULL;
+	if (keyring_name != NULL) {
+		keyring = find_keyring (keyring_name);
+		if (keyring != NULL) {
+			item = find_item (keyring, item_id);
+			if (item != NULL) {
+				access_request = access_request_from_item (item);
+				access_requests = g_list_prepend (access_requests,
+								  access_request);
+			}
+		}
+	}
+
+	*access_requests_out = access_requests;
+	g_free (keyring_name);
+	
+	return TRUE;
+	
+}
+
 static gboolean
 op_get_item_attributes_execute (GString *packet,
 				GString *result,
@@ -1851,9 +1946,6 @@ out:
 	return TRUE;
 }
 
-
-
-
 static gboolean
 op_set_item_info_or_attributes_collect (GString *packet,
 					GList **access_requests_out)
@@ -1881,7 +1973,7 @@ op_set_item_info_or_attributes_collect (GString *packet,
 			item = find_item (keyring, item_id);
 			if (item != NULL) {
 				access_request =
-					access_request_from_item (item,
+					access_request_from_item_with_secret (item,
 								  GNOME_KEYRING_ACCESS_WRITE);
 				access_requests = g_list_prepend (access_requests,
 								  access_request);
@@ -1946,7 +2038,7 @@ op_set_item_info_execute (GString *packet,
 
 	gnome_keyring_proto_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
 
-	if (type != GNOME_KEYRING_ITEM_NO_TYPE) {
+	if ((type & GNOME_KEYRING_ITEM_TYPE_MASK) != GNOME_KEYRING_ITEM_NO_TYPE) {
 		item->type = type;
 	}
 	if (item_name != NULL) {
@@ -2153,7 +2245,7 @@ op_find_execute (GString *packet,
 		access_request = l->data;
 		item = access_request->item;
 		if (item != NULL &&
-		    item->type == type &&
+		    (item->type & GNOME_KEYRING_ITEM_TYPE_MASK) == (type & GNOME_KEYRING_ITEM_TYPE_MASK) &&
 		    !item->locked &&
 		    match_attributes (item, attributes, FALSE)) {
 			if (!gnome_keyring_proto_add_utf8_string (result, item->keyring->keyring_name)) {
@@ -2209,10 +2301,10 @@ op_find_collect (GString *packet,
 		keyring = klist->data;
 		for (ilist = keyring->items; ilist != NULL; ilist = ilist->next) {
 			item = ilist->data;
-			if (item->type == type &&
+			if ((item->type & GNOME_KEYRING_ITEM_TYPE_MASK) == (type & GNOME_KEYRING_ITEM_TYPE_MASK) &&
 			    match_attributes (item, keyring->locked ? hashed : attributes, FALSE)) {
 				access_request =
-				  access_request_from_item (item, GNOME_KEYRING_ACCESS_READ);
+				  access_request_from_item_with_secret (item, GNOME_KEYRING_ACCESS_READ);
 				access_requests = g_list_prepend (access_requests,
 								  access_request);
 			}
@@ -2456,77 +2548,82 @@ launch_ask_helper (GnomeKeyringAsk *ask,
 static void
 schedule_ask (GnomeKeyringAsk *ask)
 {
-	GnomeKeyringAccessRequest *request;
+	GnomeKeyringAccessRequest *request = ask->current_request;
+	gboolean app_secret = FALSE;
+	gboolean deny = FALSE;
+	gboolean iterate = FALSE;
+
+	if(request->item) {
+		app_secret = (request->item->type & GNOME_KEYRING_ITEM_APPLICATION_SECRET) == 
+					GNOME_KEYRING_ITEM_APPLICATION_SECRET;
+	}
 
-	request = ask->current_request;
 	switch (request->request_type) {
 	case GNOME_KEYRING_ACCESS_REQUEST_KEYRING:
-		if (!launch_ask_helper (ask, ASK_KEYRING_PASSWORD)) {
-			/* no way to allow request, denying */
-			ask->access_requests = g_list_remove (ask->access_requests, request);
-			gnome_keyring_access_request_free (request);
-			/* iterate */
-			gnome_keyring_ask_iterate (ask);
-		}
+		if (!launch_ask_helper (ask, ASK_KEYRING_PASSWORD))
+			deny = iterate = TRUE;
 		break;
+
 	case GNOME_KEYRING_ACCESS_REQUEST_ITEM:
 		if (request->item->keyring->locked) {
-			if (!launch_ask_helper (ask, ASK_KEYRING_PASSWORD)) {
-				/* no way to allow request, denying */
-				ask->access_requests = g_list_remove (ask->access_requests, request);
-				gnome_keyring_access_request_free (request);
-				/* iterate */
-				gnome_keyring_ask_iterate (ask);
-			}
+			if (!launch_ask_helper (ask, ASK_KEYRING_PASSWORD)) 
+				deny = iterate = TRUE;
+
 		} else {
-			if (!launch_ask_helper (ask, ASK_ITEM_READ_WRITE_ACCESS)) {
-				/* no way to allow request, denying */
-				ask->access_requests = g_list_remove (ask->access_requests, request);
-				gnome_keyring_access_request_free (request);
-				/* iterate */
-				gnome_keyring_ask_iterate (ask);
-			}
+			/* We never prompt for simple read requests, they're either allowed or not */
+			request->granted = !app_secret;
+			deny = app_secret;
+			iterate = TRUE;
 		}
 		break;
+
+	case GNOME_KEYRING_ACCESS_REQUEST_ITEM_SECRET:
+		if (request->item->keyring->locked) {
+			if (!launch_ask_helper (ask, ASK_KEYRING_PASSWORD)) 
+				deny = iterate = TRUE;
+
+		} else if (app_secret) {
+			/* We never allow access to 'application' secrets from the wrong app */
+			deny = iterate = TRUE;
+
+		} else {
+			if (!launch_ask_helper (ask, ASK_ITEM_READ_WRITE_ACCESS)) 
+				deny = iterate = TRUE;
+		}
+		break;
+
 	case GNOME_KEYRING_ACCESS_REQUEST_NEW_KEYRING_PASSWORD:
-		if (!launch_ask_helper (ask, ASK_NEW_KEYRING_PASSWORD)) {
-			/* no way to allow request, denying */
-			ask->access_requests = g_list_remove (ask->access_requests, request);
-			gnome_keyring_access_request_free (request);
-			/* iterate */
-			gnome_keyring_ask_iterate (ask);
-		}
+		if (!launch_ask_helper (ask, ASK_NEW_KEYRING_PASSWORD)) 
+			deny = iterate = TRUE;
 		break;	
+
 	case GNOME_KEYRING_ACCESS_REQUEST_ORIGINAL_CHANGE_KEYRING_PASSWORD:
-		if (!launch_ask_helper (ask, ASK_ORIGINAL_CHANGE_KEYRING_PASSWORD)) {
-			/* no way to allow request, denying */
-			ask->access_requests = g_list_remove (ask->access_requests, request);
-			gnome_keyring_access_request_free (request);
-			/* iterate */
-			gnome_keyring_ask_iterate (ask);
-		}
+		if (!launch_ask_helper (ask, ASK_ORIGINAL_CHANGE_KEYRING_PASSWORD))
+			deny = iterate = TRUE;
 		break;
+
 	case GNOME_KEYRING_ACCESS_REQUEST_CHANGE_KEYRING_PASSWORD:
-		if (!launch_ask_helper (ask, ASK_CHANGE_KEYRING_PASSWORD)) {
-			/* no way to allow request, denying */
-			ask->access_requests = g_list_remove (ask->access_requests, request);
-			gnome_keyring_access_request_free (request);
-			/* iterate */
-			gnome_keyring_ask_iterate (ask);
-		}
+		if (!launch_ask_helper (ask, ASK_CHANGE_KEYRING_PASSWORD))
+			deny = iterate = TRUE;
 		break;
+
 	case GNOME_KEYRING_ACCESS_REQUEST_DEFAULT_KEYRING:
-		if (!launch_ask_helper (ask, ASK_DEFAULT_KEYRING)) {
-			/* no way to allow request, denying */
-			ask->access_requests = g_list_remove (ask->access_requests, request);
-			gnome_keyring_access_request_free (request);
-			/* iterate */
-			gnome_keyring_ask_iterate (ask);
-		}
+		if (!launch_ask_helper (ask, ASK_DEFAULT_KEYRING))
+			deny = iterate = TRUE;
 		break;
+
 	default:
 		g_assert_not_reached ();
 	}
+
+	if (deny) {
+		ask->access_requests = g_list_remove (ask->access_requests, request);
+		gnome_keyring_access_request_free (request);
+	}
+
+	if (iterate) 
+		gnome_keyring_ask_iterate (ask);
+
 }
 
 static gboolean
@@ -2669,14 +2766,15 @@ GnomeKeyringOperationImplementation keyring_ops[] = {
 	{ op_find_collect, op_find_execute }, /* FIND */
 	{ op_create_item_collect, op_create_item_execute}, /* CREATE_ITEM */
 	{ op_delete_item_collect, op_delete_item_execute}, /* DELETE_ITEM */
-	{ op_get_item_info_or_attributes_collect, op_get_item_info_execute}, /* GET_ITEM_INFO */
+	{ op_get_item_info_collect, op_get_item_info_execute}, /* GET_ITEM_INFO */
 	{ op_set_item_info_or_attributes_collect, op_set_item_info_execute}, /* SET_ITEM_INFO */
-	{ op_get_item_info_or_attributes_collect, op_get_item_attributes_execute}, /* GET_ITEM_ATTRIBUTES */
+	{ op_get_item_acl_or_attributes_collect, op_get_item_attributes_execute}, /* GET_ITEM_ATTRIBUTES */
 	{ op_set_item_info_or_attributes_collect, op_set_item_attributes_execute}, /* SET_ITEM_ATTRIBUTES */
-	{ op_get_item_info_or_attributes_collect, op_get_item_acl_execute}, /* GET_ITEM_ACL */
+	{ op_get_item_acl_or_attributes_collect, op_get_item_acl_execute}, /* GET_ITEM_ACL */
 	{ op_set_item_info_or_attributes_collect, op_set_item_acl_execute}, /* SET_ITEM_ACL */
 	{ op_change_keyring_password_collect, op_change_keyring_password_execute }, /*CHANGE_KEYRING_PASSWORD*/
  	{ NULL, op_set_daemon_display_execute}, /* SET_DAEMON_DISPLAY */
+	{ op_get_item_info_collect, op_get_item_info_execute}, /* GET_ITEM_INFO_PARTIAL */
 };
 
 static RETSIGTYPE
