@@ -56,7 +56,7 @@ typedef enum {
 	GNOME_KEYRING_ASK_RESPONSE_FAILURE,
 	GNOME_KEYRING_ASK_RESPONSE_DENY,
 	GNOME_KEYRING_ASK_RESPONSE_ALLOW_ONCE,
-	GNOME_KEYRING_ASK_RESPONSE_ALLOW_FOREVER,
+	GNOME_KEYRING_ASK_RESPONSE_ALLOW_FOREVER
 } GnomeKeyringAskResponse;
 
 
